var $ = require('./jquery');

var supportTouch = ('ontouchstart' in window),
	scrollEvent = "touchmove scroll",
	touchStartEvent = supportTouch ? "touchstart" : "mousedown",
	touchStopEvent = supportTouch ? "touchend" : "mouseup",
	touchMoveEvent = supportTouch ? "touchmove" : "mousemove";
var simulatorEventtype = ['mousedown', 'mousemove', 'mouseup', 'mouseup'];
$.each('touchstart touchmove touchend touchcancel'.split(" "), function(i, name) {
	$.event.special[name] = {
		setup: function() {
			if (supportTouch) return false;
			var elem = $(this);
			elem.bind(simulatorEventtype[i], touch_handler);
		},
		teardown: function() {
			if (supportTouch) return false;
			var elem = $(this);
			elem.unbind(simulatorEventtype[i], touch_handler);
		}
	};

	function touch_handler(e) {
		if (e.which == 3) {
			return;
		}
		var touchsEvent = [{
			pageX: e.pageX,
			pageY: e.pageY
		}];
		e.originalEvent = {
			touches: touchsEvent,
			changedTouches: touchsEvent,
			pageX: e.pageX,
			pageY: e.pageY
		};
		e.type = name;
		$(e.srcElement).trigger(e);
	}
});

$.event.special.tap = {
	setup: function() {
		var thisObject = this,
			$this = $(thisObject);

		$this.bind(touchStartEvent, function(event) {
			if (event.which && event.which !== 1 ||
			//check if event fired once already by a device that fires both mousedown and touchstart (while supporting both events)
			$this.data("prevEvent") && $this.data("prevEvent") !== event.type) {
				return false;
			}

			//save event type so only this type is let through for a temp duration,
			//allowing quick repetitive taps but not duplicative events
			$this.data("prevEvent", event.type);
			setTimeout(function() {
				$this.removeData("prevEvent");
			}, 800);

			var moved = false,
				touching = true,
				origTarget = event.target,
				origEvent = event.originalEvent,
				origPos = event.type == "touchstart" ? [origEvent.touches[0].pageX, origEvent.touches[0].pageY] : [event.pageX, event.pageY],
				originalType, timer;

			function moveHandler(event) {
				if (event.type == "scroll") {
					moved = true;
					return;
				}
				var newPageXY = event.type == "touchmove" ? event.originalEvent.touches[0] : event;
				if ((Math.abs(origPos[0] - newPageXY.pageX) > 10) || (Math.abs(origPos[1] - newPageXY.pageY) > 10)) {
					moved = true;
				}
			}

			timer = setTimeout(function() {
				if (touching && !moved) {
					originalType = event.type;
					event.type = "taphold";
					$.event.handle.call(thisObject, event);
					event.type = originalType;
				}
			}, 750);

			//scroll now cancels tap
			$(window).one("scroll", moveHandler);

			$this.bind(touchMoveEvent, moveHandler).one(touchStopEvent, function(event) {
				$this.unbind(touchMoveEvent, moveHandler);
				$(window).unbind("scroll", moveHandler);
				clearTimeout(timer);
				touching = false;

				/* ONLY trigger a 'tap' event if the start target is
 * the same as the stop target.
 */
				if (!moved && (origTarget == event.target)) {
					originalType = event.type;
					event.type = "tap";
					$.event.handle.call(thisObject, event);
					event.type = originalType;
				}
			});
		});
	}
};